Package: PathoStat
Type: Package
Title: PathoStat Statistical Microbiome Analysis Package
Version: 1.38.0
Date: 2020-03-27
Author: Solaiappan Manimaran <manimaran_1975@hotmail.com>, Matthew
        Bendall <bendall@gwmail.gwu.edu>, Sandro Valenzuela Diaz
        <sandrolvalenzuelad@gmail.com>, Eduardo Castro
        <castronallar@gmail.com>, Tyler Faits <tfaits@gmail.com>, Yue
        Zhao <jasonzhao0307@gmail.com>, Anthony Nicholas Federico
        <anfed@bu.edu>, W. Evan Johnson <wej@bu.edu>
Maintainer: Solaiappan Manimaran <manimaran_1975@hotmail.com>, Yue Zhao
 <jasonzhao0307@gmail.com>
Description: The purpose of this package is to perform Statistical
        Microbiome Analysis on metagenomics results from sequencing
        data samples. In particular, it supports analyses on the
        PathoScope generated report files. PathoStat provides various
        functionalities including Relative Abundance charts, Diversity
        estimates and plots, tests of Differential Abundance, Time
        Series visualization, and Core OTU analysis.
URL: https://github.com/mani2012/PathoStat
BugReports: https://github.com/mani2012/PathoStat/issues
License: GPL (>= 2)
Depends: R (>= 3.5)
Imports: limma, corpcor,matrixStats, reshape2, scales, ggplot2,
        rentrez, DT, tidyr, plyr, dplyr, phyloseq, shiny, stats,
        methods, XML, graphics, utils, BiocStyle, edgeR, DESeq2,
        ComplexHeatmap, plotly, webshot, vegan, shinyjs, glmnet,
        gmodels, ROCR, RColorBrewer, knitr, devtools, ape
Collate: 'pathoStat.R' 'utils.R' 'taxonomy.R' 'biomarker.R'
        'allClasses.R' 'visualization.R' 'differentialAnalysis.R'
biocViews: Microbiome, Metagenomics, GraphAndNetwork, Microarray,
        PatternLogic, PrincipalComponent, Sequencing, Software,
        Visualization, RNASeq, ImmunoOncology
RoxygenNote: 6.1.1
Encoding: UTF-8
Suggests: rmarkdown, testthat
VignetteBuilder: knitr
Config/pak/sysreqs: cmake libfontconfig1-dev libfreetype6-dev
        libfribidi-dev git libglpk-dev make libharfbuzz-dev
        libmagick++-dev gsfonts libgit2-dev libicu-dev libjpeg-dev
        libpng-dev libtiff-dev libuv1-dev libwebp-dev libxml2-dev
        libssl-dev perl libx11-dev zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:44:26 UTC
RemoteUrl: https://github.com/bioc/PathoStat
RemoteRef: RELEASE_3_23
RemoteSha: c77c976c2d70df3e8970018d26de61662d3c13a0
NeedsCompilation: no
Packaged: 2026-04-29 23:47:15 UTC; root
Built: R 4.6.0; ; 2026-04-30 01:19:57 UTC; unix
