# Generated by roxygen2: do not edit by hand

export(Bootstrap_LOOCV_LR_AUC)
export(Chisq_Test_Pam)
export(Fisher_Test_Pam)
export(GET_PAM)
export(LOOAUC_simple_multiple_noplot_one_df)
export(LOOAUC_simple_multiple_one_df)
export(TranslateIdToTaxLevel)
export(Wilcox_Test_df)
export(findRAfromCount)
export(findTaxonMat)
export(findTaxonomy)
export(findTaxonomy300)
export(getShinyInput)
export(getShinyInputCombat)
export(getShinyInputOrig)
export(getSignatureFromMultipleGlmnet)
export(grepTid)
export(loadPathoscopeReports)
export(loadPstat)
export(log2CPM)
export(pathostat1)
export(percent)
export(phyloseq_to_edgeR)
export(plotPCAPlotly)
export(plotPCoAPlotly)
export(readPathoscopeData)
export(runPathoStat)
export(savePstat)
export(setShinyInput)
export(setShinyInputCombat)
export(setShinyInputOrig)
export(summarizeTable)
exportClasses(PathoStat)
import(BiocStyle)
import(DT)
import(ape)
import(corpcor)
import(devtools)
import(edgeR)
import(ggplot2)
import(glmnet)
import(graphics)
import(knitr)
import(limma)
import(matrixStats)
import(methods)
import(phyloseq)
import(rentrez)
import(reshape2)
import(stats)
import(tidyr)
import(vegan)
import(webshot)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,estimateDispersions)
importFrom(DESeq2,nbinomWaldTest)
importFrom(DESeq2,results)
importFrom(DESeq2,sizeFactors)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(dplyr,add_rownames)
importFrom(dplyr,mutate_each)
importFrom(dplyr,select_)
importFrom(gmodels,ci)
importFrom(plotly,plot_ly)
importFrom(plyr,ddply)
importFrom(plyr,numcolwise)
importFrom(scales,percent_format)
importFrom(shiny,getShinyOption)
importFrom(shiny,runApp)
importFrom(shiny,shinyOptions)
importFrom(shinyjs,useShinyjs)
importFrom(utils,browseURL)
importFrom(utils,read.table)
